@@ -177,16 +177,11 @@ impl ProcTable {
         for pid in self.processes.keys() {
             if Path::new(&format!("/proc/{}", pid)).is_dir() {
                 let mut pid = *pid;
-                loop {
-                    if let Some(proc) = self.processes.get(&pid) {
-                        prune.remove(&pid);
-                        if pid == 0 || pid == proc.ppid {
-                            break
-                        }
-                        pid = proc.ppid;
-                    } else {
-                        break;
+                while let Some(proc) = self.processes.get(&pid) {
+                    if pid <= 1 || !prune.remove(&pid) {
+                        break
                     }
+                    pid = proc.ppid;
                 }
             }
         }
