@@ -125,6 +125,8 @@ pub enum Value {
     /// `a0`, `a1`, `a2` … -> `ARGV`
     List(Vec<Value>),
     StringifiedList(Vec<Value>),
+    /// Key/Value map, used in ENV (environment variables) list
+    Map(Vec<(Range<usize>,Range<usize>)>),
     /// Values generated in parse() from unquoted Str values
     ///
     /// For example, `SYSCALL` / `a0` etc are interpreted as
@@ -134,7 +136,7 @@ pub enum Value {
 
 /// List of [`Key`]/[`Value`] pairs, that are, for the most part,
 /// stored offsets into the raw log line.
-#[derive(Clone)]
+#[derive(Default,Clone)]
 pub struct Record {
     pub elems: Vec<(Key, Value)>,
     pub raw: Vec<u8>,
@@ -176,7 +178,7 @@ impl<'a> Record {
                 Value::Str(r,q) => Value::Str(r.offset(rawlen),q),
                 Value::Empty => Value::Empty,
                 Value::Number(n) => Value::Number(n),
-                Value::Segments(_) | Value::List(_) | Value::StringifiedList(_) =>
+                Value::Segments(_) | Value::List(_) | Value::StringifiedList(_) | Value::Map(_) =>
                     panic!("extend after normalize?"),
             }
         )).collect::<Vec<_>>())
@@ -191,6 +193,13 @@ impl<'a> Record {
         }
         None
     }
+
+    /// Add a byte string to a record.
+    pub fn put(&mut self, s: &[u8]) -> Range<usize> {
+        let b = self.raw.len();
+        self.raw.extend(s);
+        b .. b+s.len()
+    }
 }
 
 impl<'a> IntoIterator for &'a Record {
@@ -275,6 +284,7 @@ impl<'a> TryFrom<RValue<'a>> for Vec<u8> {
             }
             Value::Number(_) => Err("Won't convert number to string".into()),
             Value::List(_) | Value::StringifiedList(_) => Err("Can't convert list to scalarr".into()),
+            Value::Map(_) => Err("Can't convert map to scalar".into()),
         }
     }
 }
@@ -282,15 +292,16 @@ impl<'a> TryFrom<RValue<'a>> for Vec<u8> {
 impl<'a> TryFrom<RValue<'a>> for Vec<Vec<u8>> {
     type Error = Box<dyn StdError>;
     fn try_from(value: RValue<'a>) -> Result<Self, Self::Error> {
-        if let Value::List(values) = value.value {
-            let mut rv = Vec::new();
-            for v in values {
-                let s = Vec::try_from(RValue{value: &v, raw: &value.raw})?;
-                rv.push(s);
+        match value.value {
+            Value::List(values) | Value::StringifiedList(values) => {
+                let mut rv = Vec::with_capacity(values.len());
+                for v in values {
+                    let s = Vec::try_from(RValue{value: &v, raw: &value.raw})?;
+                    rv.push(s);
+                }
+                Ok(rv)
             }
-            Ok(rv)
-        } else {
-            Err("not a list".into())
+            _ => Err("not a list".into())
         }
     }
 }
@@ -330,6 +341,7 @@ impl<'a> Debug for RValue<'a> {
                         Value::Empty     => panic!("list can't contain empty value"),
                         Value::HexStr(_) => panic!("list can't contain hex string"),
                         Value::List(_) | Value::StringifiedList(_) => panic!("list can't contain list"),
+                        Value::Map(_) => panic!("list can't contain map"),
                         Value::Number(_) => panic!("List can't contain number"),
                     }
                 }
@@ -354,11 +366,25 @@ impl<'a> Debug for RValue<'a> {
                         Value::Empty     => panic!("list can't contain empty value"),
                         Value::HexStr(_) => panic!("list can't contain hex string"),
                         Value::List(_) | Value::StringifiedList(_) => panic!("list can't contain list"),
+                        Value::Map(_) => panic!("List can't contain mapr"),
                         Value::Number(_) => panic!("List can't contain number"),
                     }
                 }
                 write!(f, ">")
             }
+            Value::Map(vs) => {
+                write!(f, "Map:<")?;
+                for (n, v) in vs.iter().enumerate() {
+                    if n > 0 {
+                        write!(f, " ")?;
+                    }
+                    write!(f, "{}={}",
+                           String::from_utf8_lossy(&self.raw[v.0.clone()]),
+                           String::from_utf8_lossy(&self.raw[v.1.clone()]))?;
+                }
+                write!(f, ">")?;
+                unimplemented!();
+            }
             Value::Number(n) => {
                 write!(f, "{:?}", n)
             }
@@ -414,6 +440,14 @@ impl<'a> Serialize for RValue<'a> {
                     Number::Oct(n) => s.serialize_str(&format!("0o{:o}", n)),
                 }
             }
+            Value::Map(vs) => {
+                let mut map = s.serialize_map(Some(vs.len()))?;
+                for v in vs {
+                    map.serialize_key(&self.raw[v.0.clone()].to_quoted_string())?;
+                    map.serialize_value(&self.raw[v.1.clone()].to_quoted_string())?;
+                }
+                map.end()
+            }
             Value::HexStr(_r) => {
                 Err(S::Error::custom("can't serialize untreated hex string"))
             }
